/*
  This file is part of Media_Controls -- the free invoicing software
  Copyright (C) 2012 Anton Katsarov <anton@katsarov.org>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

const St = imports.gi.St;
const Mainloop = imports.mainloop;
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;
const Lang = imports.lang;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Shell = imports.gi.Shell;

const Me = imports.ui.extensionSystem.extensions['Media_Controls@katsarov.org'];
const DBusIface = Me.dbus;

const Gettext = imports.gettext.domain('gnome-shell-extension-media-controls');

let text;
let players;
let Player;
let Players;

function Players() {
    this._init.apply(this, arguments);
}

Players.prototype = {
    _init: function(menu) {
        // players list
        this._players = {};
        // watch list
        this._watch = [];

        // watch players
        for (var p = 0; p < players.length; p++) {
            this._watch.push(Gio.DBus.session.watch_name('org.mpris.MediaPlayer2.'+players[p],
                                                         Gio.BusNameWatcherFlags.NONE,
                                                         Lang.bind(this, this._addPlayer),
                                                         Lang.bind(this, this._removePlayer)
            ));
        }
    },
    _addPlayer: function (conn, owner) {
        this._players[owner] = new Player(owner);
    },
    _removePlayer: function (conn, owner) {
        if (this._players[owner]) {
            this._players[owner].destroy();
            delete this._players[owner];
        }
    },
    destroy: function() {
        for (let w = 0; w < this._watch.length; w++) {
            Gio.DBus.session.unwatch_name(this._watch[w]);
        }
        for (let owner in this._players) {
            this._players[owner].destroy();
        }
    }
};

function Player() {
    this._init.apply(this, arguments);
}

Player.prototype = {
    _init: function(owner) {
        this._mediaServerPlayer = new DBusIface.MediaServer2Player(owner);
        this._prop = new DBusIface.Prop(owner);

	this._button_fw = new St.Bin({ style_class: 'panel-button',
				       reactive: true,
				       can_focus: true,
				       x_fill: true,
				       y_fill: false,
				       track_hover: true });
	this._icon_fw = new St.Icon({ icon_name: 'media-skip-forward',
				      icon_type: St.IconType.SYMBOLIC,
    				      icon_size: 16,
				      style_class: 'media-skip-froward' });

	this._button_fw.set_child(this._icon_fw);

	this._button_play = new St.Bin({ style_class: 'panel-button',
					 reactive: true,
					 can_focus: true,
					 x_fill: true,
					 y_fill: false,
					 track_hover: true });
	this._icon_play = new St.Icon({ icon_name: 'media-playback-start',
					icon_type: St.IconType.SYMBOLIC,
					icon_size: 16,
					style_class: 'media-playback-start' });

	this._button_play.set_child(this._icon_play);

	this._button_bw = new St.Bin({ style_class: 'panel-button',
				       reactive: true,
				       can_focus: true,
				       x_fill: true,
				       y_fill: false,
				       track_hover: true });
	this._icon_bw = new St.Icon({ icon_name: 'media-skip-backward',
				      icon_type: St.IconType.SYMBOLIC,
    				      icon_size: 16,
				      style_class: 'media-skip-backward' });

	this._button_bw.set_child(this._icon_bw);

	this._button_logo = new St.Bin({ style_class: 'panel-button',
				       reactive: true,
				       can_focus: true,
				       x_fill: true,
				       y_fill: false,
				       track_hover: true });
	this._icon_logo = new St.Icon({ icon_name: 'audio-x-generic',
				      icon_type: St.IconType.FULLCOLOR,
    				      icon_size: 16,
				      style_class: 'media-skip-backward' });

        this._name = owner.split('.')[3];
	if (this._name == "rhythmbox3") this._name = "rhythmbox";
        if (this._name)
            this._icon_logo.icon_name = this._name;
	this._button_logo.set_child(this._icon_logo);

        this._getStatus();

        this._prop.connect('PropertiesChanged', Lang.bind(this, function(sender, iface, value) {
            if (value["PlaybackStatus"])
                this._setStatus(iface, value["PlaybackStatus"]);
        }));


        this._appsys = Shell.AppSystem.get_default();
        this._appobj = this._appsys.lookup_app(this._name + ".desktop");
	this._button_play.connect('button-press-event', Lang.bind(this, function () { this._mediaServerPlayer.PlayPauseRemote() } ));
	this._button_bw.connect('button-press-event', Lang.bind(this, function () { this._mediaServerPlayer.PreviousRemote() } ));
	this._button_fw.connect('button-press-event', Lang.bind(this, function () { this._mediaServerPlayer.NextRemote() } ));
	this._button_logo.connect('button-press-event', Lang.bind(this, function () {
            Mainloop.timeout_add(100, Lang.bind(this, function () {
		windowm = this._appobj.get_windows()[0];
		Main.activateWindow(windowm);
                }));
        }));


	Main.panel._rightBox.insert_actor(this._button_fw, 0);
	Main.panel._rightBox.insert_actor(this._button_play, 0);
	Main.panel._rightBox.insert_actor(this._button_bw, 0);
	Main.panel._rightBox.insert_actor(this._button_logo, 0);
    },
    _getStatus: function () {
        this._mediaServerPlayer.getPlaybackStatus(Lang.bind(this,
            this._setStatus
        ));
    },
    _setStatus: function (sender, status) {
        if (status != this._status) {
            this._status = status;
            if (this._status == "Playing") {
                this._icon_play.icon_name = "media-playback-pause";
            }
            else if (this._status == "Paused") {
                this._icon_play.icon_name = "media-playback-start";
            }
            else if (this._status == "Stopped") {
                this._icon_play.icon_name = "media-playback-start";
            }
        }
    },
    destroy: function() {
	Main.panel._rightBox.remove_actor(this._button_fw);
	Main.panel._rightBox.remove_actor(this._button_play);
	Main.panel._rightBox.remove_actor(this._button_bw);
	Main.panel._rightBox.remove_actor(this._button_logo);
    }

};

function init(metadata) {
    imports.gettext.bindtextdomain('gnome-shell-extension-media-controls', GLib.build_filenamev([metadata.path, 'locale']));
    players = metadata.players;

}

function enable() {
    Players = new Players();
}
function disable() {
    Players.destroy();
}
